package net.sf.mgp.unclasses;

/*
 * Copyright 2008 MGP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Collection;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;

abstract class AbstractAsmDependencyResolver implements DependencyResolver {
	private static final InvocationHandler EMPTY_INVOCATION_HANDLER = new InvocationHandler() {
		public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
			return null;
		}
	};

	protected ClassVisitor createEmptyClassVisitor() {
		return (ClassVisitor) createEmptyProxy(ClassVisitor.class);
	}

	protected MethodVisitor createEmptyMethodVisitor() {
		return (MethodVisitor) createEmptyProxy(MethodVisitor.class);
	}

	protected Object createEmptyProxy(Class requiredInterface) {
		return Proxy.newProxyInstance(getClass().getClassLoader(), new Class[] { requiredInterface }, EMPTY_INVOCATION_HANDLER);
	}

	public Collection getDependencyNames(File classpathRoot, String classFile) {
		try {
			InputStream is = new FileInputStream(new File(classpathRoot, classFile));
			try {
				ClassReader reader = new ClassReader(is);
				return getDependencyNames(reader);
			} finally {
				is.close();
			}
		} catch (IOException e) {
			// TODO replace with exception chaining asap switched to java5
			throw new IllegalArgumentException("Failed to load class: " + e);
		}
	}

	protected abstract Collection getDependencyNames(ClassReader reader);
}
