package net.sf.mgp.unclasses;

/*
 * Copyright 2008 MGP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.util.ClassPath;
import org.apache.bcel.util.Repository;
import org.apache.bcel.util.SyntheticRepository;

/**
 * Implementation of {@link DependencyResolver} resolving class dependencies. Implemented using Apache BCEL (http://jakarta.apache.org/bcel/).
 * 
 * @plexus.component role="net.sf.mgp.unclasses.DependencyResolver" role-hint="classes"
 * 
 * @author Maksim Gordienko
 * 
 */
public class BcelClassDependencyResolver implements DependencyResolver {

	public Collection getDependencyNames(File classpathRoot, String classFile) {
		try {
			Repository r = SyntheticRepository.getInstance(new ClassPath(classpathRoot.getCanonicalPath()));
			String className = Utils.getClassNameFromFileName(classFile);
			JavaClass jc = r.loadClass(className);

			Constant[] cs = jc.getConstantPool().getConstantPool();
			List dependencies = new ArrayList();
			for (int i = 0; i < cs.length; i++) {
				Constant c = cs[i];
				String dependentClassName = null;
				if (c instanceof ConstantClass) {
					dependentClassName = jc.getConstantPool().constantToString(c);
				} else if (c instanceof ConstantUtf8) {
					String utfConstant = jc.getConstantPool().constantToString(c);
					if (Utils.isClassDescriptor(utfConstant)) {
						dependentClassName = Utils.getClassNameFromDescriptor(utfConstant);
					}
				}
				if (dependentClassName != null) {
					if (!className.equals(dependentClassName)) {
						dependencies.add(dependentClassName);
					}
				}
			}
			return dependencies;
		} catch (IOException e) {
			// TODO replace with exception chaining asap switched to java5
			throw new IllegalArgumentException("Failed to load class: " + e);
		} catch (ClassNotFoundException e) {
			// TODO replace with exception chaining asap switched to java5
			throw new IllegalArgumentException("Failed to load class: " + e);
		}
	}

}
