package net.sf.mgp.unclasses;

/*
 * Copyright 2008 MGP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

class GraphNode {
	private final Set to;
	private final Set from;
	private final String value;
	private boolean projectNode;

	public GraphNode(String value) {
		this.value = value;
		this.to = new HashSet();
		this.from = new HashSet();
	}

	public void addEdge(GraphNode toNode) {
		to.add(toNode);
		toNode.from.add(this);
	}

	public void remove() {
		Set dependants = fillDependants(new HashSet(), false);
		for (Iterator it = dependants.iterator(); it.hasNext();) {
			GraphNode node = (GraphNode) it.next();
			node.to.clear();
			for (Iterator it2 = node.from.iterator(); it2.hasNext();) {
				GraphNode fromNode = (GraphNode) it2.next();
				fromNode.to.remove(node);
			}
		}
	}

	private Set fillDependants(Set dependants, boolean exclusive) {
		if (dependants.add(this)) {
			for (Iterator it = to.iterator(); it.hasNext();) {
				GraphNode child = (GraphNode) it.next();
				if (!(exclusive && child.from.size() != 1)) {
					child.fillDependants(dependants, exclusive);
				}
			}
		}
		return dependants;
	}

	public void setProjectNode() {
		this.projectNode = true;
	}

	public boolean isUnreferenced() {
		return from.isEmpty();
	}

	public String getValue() {
		return value;
	}

	public Collection getExclusiveProjectDependants() {
		Set result = fillDependants(new HashSet(), true);
		result.remove(this);
		for (Iterator it = result.iterator(); it.hasNext();) {
			GraphNode dependant = (GraphNode) it.next();
			if (!dependant.projectNode) {
				it.remove();
			}
		}
		return result;
	}
}
